@@ -1,21 +1,21 @@
 /*
- * Copyright (C) 2001  Internet Software Consortium.
+ * Copyright (C) 2004  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2001, 2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM
- * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
- * INTERNET SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
- * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: rndc-confgen.c,v 1.9.2.1 2001/09/06 00:14:16 gson Exp $ */
+/* $Id: rndc-confgen.c,v 1.9.2.8 2004/09/28 07:18:28 marka Exp $ */
 
 #include <config.h>
 
@@ -66,7 +66,7 @@ Usage:\n\
 [-s addr] [-t chrootdir] [-u user]\n\
   -a:		generate just the key clause and write it to keyfile (%s)\n\
   -b bits:	from 1 through 512, default %d; total length of the secret\n\
-  -c keyfile:	specify a alterate keyfile (requires -a)\n\
+  -c keyfile:	specify an alternate key file (requires -a)\n\
   -k keyname:	the name as it will be used  in named.conf and rndc.conf\n\
   -p port:	the port named will listen on and rndc will connect to\n\
   -r randomfile: a file containing random data\n\
@@ -90,31 +90,21 @@ write_key_file(const char *keyfile, const char *user,
 	FILE *fd;
 
 	fd = safe_create(keyfile);
-	if (fd == NULL) {
-		fprintf(stderr, "unable to create \"%s\"\n", keyfile);
-		return;
-	}
+	if (fd == NULL)
+		fatal( "unable to create \"%s\"\n", keyfile);
 	if (user != NULL) {
-		if (set_user(fd, user) == -1) {
-			fprintf(stderr, "unable to set file owner\n");
-			fclose(fd);
-			return;
-		}
+		if (set_user(fd, user) == -1)
+			fatal("unable to set file owner\n");
 	}
 	fprintf(fd, "key \"%s\" {\n\talgorithm hmac-md5;\n"
 		"\tsecret \"%.*s\";\n};\n", keyname,
 		(int)isc_buffer_usedlength(secret),
 		(char *)isc_buffer_base(secret));
 	fflush(fd);
-	if (ferror(fd)) {
-		fprintf(stderr, "write to %s failed\n", keyfile);
-		fclose(fd);
-		return;
-	}
-	if (fclose(fd)) {
-		fprintf(stderr, "fclose(%s) failed\n", keyfile);
-		return;
-	}
+	if (ferror(fd))
+		fatal("write to %s failed\n", keyfile);
+	if (fclose(fd))
+		fatal("fclose(%s) failed\n", keyfile);
 }
 
 int
@@ -139,7 +129,8 @@ main(int argc, char **argv) {
 	int keysize;
 	int entropy_flags = 0;
 	int open_keyboard = ISC_ENTROPY_KEYBOARDMAYBE;
-	struct in_addr addr;
+	struct in_addr addr4_dummy;
+	struct in6_addr addr6_dummy;
 	char *chrootdir = NULL;
 	char *user = NULL;
 	isc_boolean_t keyonly = ISC_FALSE;
@@ -197,9 +188,9 @@ main(int argc, char **argv) {
 			break;
 		case 's':
 			serveraddr = isc_commandline_argument;
-			if (inet_aton(serveraddr, &addr) == 0)
+			if (inet_pton(AF_INET, serveraddr, &addr4_dummy) != 1 &&
+			    inet_pton(AF_INET6, serveraddr, &addr6_dummy) != 1)
 				fatal("-s should be an IPv4 or IPv6 address");
-				
 			break;
 		case 't':
 			chrootdir = isc_commandline_argument;
@@ -279,11 +270,10 @@ main(int argc, char **argv) {
 			char *buf;
 			len = strlen(chrootdir) + strlen(keyfile) + 2;
 			buf = isc_mem_get(mctx, len);
-			if (buf != NULL) {
-				fprintf(stderr, "isc_mem_get(%d) failed\n", len);
-				goto cleanup;
-			}
-			snprintf(buf, len, "%s/%s", chrootdir, keyfile);
+			if (buf == NULL)
+				fatal("isc_mem_get(%d) failed\n", len);
+			snprintf(buf, len, "%s%s%s", chrootdir,
+				 (*keyfile != '/') ? "/" : "", keyfile);
 			
 			write_key_file(buf, user, keyname, &key_txtbuffer);
 			isc_mem_put(mctx, buf, len);
@@ -324,7 +314,6 @@ options {\n\
 		       serveraddr, port, serveraddr, keyname);
 	}
 
- cleanup:
 	if (show_final_mem)
 		isc_mem_stats(mctx, stderr);
 
